/*
 * BeanShell.java - Bean shell
 * Copyright (C) 1999, 2000 Slava Pestov
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

import bsh.*;
import java.io.File;
import org.gjt.sp.jedit.*;
import org.gjt.sp.util.Log;

public class BeanShell implements Shell
{
	public void printInfoMessage(Output output)
	{
		output.printInfo(jEdit.getProperty("beanshell.info"));
	}

	public void execute(View view, String command, Output output)
	{
		//BeanShellPlugin.clearErrors();

		interp.setVariable("view",view);
		interp.setVariable("output",output);
		interp.setVariable("shell",this);

		try
		{
			interp.eval(command);
			exitCode = true;
		}
		catch(Throwable t)
		{
			Log.log(Log.ERROR,this,t);
			if(output != null)
				output.printError(t.toString());
			else
			{
				String[] args = { t.toString() };
				GUIUtilities.error(view,"bsh-error",args);
			}
			exitCode = false;
		}
	}

	public void source(View view, String file, Output output)
	{
		//BeanShellPlugin.clearErrors();

		interp.setVariable("view",view);
		interp.setVariable("output",output);
		interp.setVariable("shell",this);

		Buffer buffer = jEdit.getBuffer(file);

		try
		{
			if(buffer != null)
				interp.eval(buffer.getText(0,buffer.getLength()));
			else
				interp.source(file);
			exitCode = true;
		}
		catch(Throwable t)
		{
			Log.log(Log.ERROR,this,t);
			if(output != null)
				output.printError(t.toString());
			else
			{
				String[] args = { t.toString() };
				GUIUtilities.error(view,"bsh-error",args);
			}
			exitCode = false;
		}
	}

	public void stop() {}

	public boolean waitFor()
	{
		return exitCode;
	}

	public Interpreter getInterpreter()
	{
		return interp;
	}

	// package-private members
	void init()
	{
		initInterp();

		File beanShellDirectory = new File(jEdit.getSettingsDirectory(),"beanshell");
		if(!beanShellDirectory.exists())
			beanShellDirectory.mkdir();
		String[] files = beanShellDirectory.list();

		if(files != null)
		{
			for(int i = 0; i < files.length; i++)
			{
				if(!files[i].toLowerCase().endsWith(".bsh"))
					continue;

				String path = MiscUtilities.constructPath(
					beanShellDirectory.getPath(),files[i]);
				Log.log(Log.MESSAGE,this,"Loading " + path);
				source(null,path,null);
			}
		}
	}

	Object method(NameSpace namespace, String name)
	{
		// XXX the next line could cause some compilers to spew
		Object method = (Object)namespace.getMethod(name);
		if(method == null)
			return null;

		try
		{
			return interp.eval(name + "()",namespace);
		}
		catch(EvalError e)
		{
			Log.log(Log.ERROR,this,e);
			String[] args = { e.toString() };
			GUIUtilities.error(null,"bsh-error",args);
		}

		return null;
	}

	Object method(NameSpace namespace, String name, Object[] args)
	{
		// XXX the next line could cause some compilers to spew
		Object method = (Object)namespace.getMethod(name);
		if(method == null)
			return null;

		// XXX dirty hack. There is no way (as far as I know)
		// to pass an array of parameters to eval().
		StringBuffer command = new StringBuffer(name);
		command.append("(");
		for(int i = 0; i < args.length; i++)
		{
			interp.setVariable("_arg_" + i,args[i]);
			if(i != 0)
				command.append(',');
			command.append("_arg_");
			command.append(i);
		}
		command.append(")");

		try
		{
			return interp.eval(command.toString(),namespace);
		}
		catch(EvalError e)
		{
			Log.log(Log.ERROR,this,e);
			String[] _args = { e.toString() };
			GUIUtilities.error(null,"bsh-error",_args);
		}

		return null;
	}

	// private members
	private Interpreter interp;
	private boolean exitCode;

	private synchronized void initInterp()
	{
		Log.log(Log.MESSAGE,this,"BeanShell plugin "
			+ jEdit.getProperty("plugin.BeanShellPlugin.version")
			+ " initializing...");

		interp = new Interpreter();

		Log.log(Log.MESSAGE,this,"Loading jedit.bsh from JAR file");

		try
		{
			interp.eval(getClass().getResourceAsStream("/jedit.bsh"));
		}
		catch(Throwable t)
		{
			Log.log(Log.ERROR,this,"Error loading jedit.bsh from JAR file:");
			Log.log(Log.ERROR,this,t);
		}
	}
}
